@@ -87,7 +87,7 @@ typedef struct WriteBuffer WriteBuffer;
 **   an aligned write() of an integer number of 512 byte regions, then
 **   option (3) above is never selected. Instead, each 512 byte region
 **   is either correctly written or left completely untouched. Similar
-**   logic governs the behavior if any of the other ATOMICXXX flags
+**   logic governs the behaviour if any of the other ATOMICXXX flags
 **   is set.
 **
 **   If either the IOCAP_SAFEAPPEND or IOCAP_SEQUENTIAL flags are set
@@ -127,15 +127,14 @@ struct CrashFile {
   const sqlite3_io_methods *pMethod;   /* Must be first */
   sqlite3_file *pRealFile;             /* Underlying "real" file handle */
   char *zName;
-  int flags;                           /* Flags the file was opened with */
 
   /* Cache of the entire file. This is used to speed up OsRead() and 
   ** OsFileSize() calls. Although both could be done by traversing the
   ** write-list, in practice this is impractically slow.
   */
-  u8 *zData;                           /* Buffer containing file contents */
+  int iSize;                           /* Size of file in bytes */
   int nData;                           /* Size of buffer allocated at zData */
-  i64 iSize;                           /* Size of file in bytes */
+  u8 *zData;                           /* Buffer containing file contents */
 };
 
 struct CrashGlobal {
@@ -156,32 +155,6 @@ static CrashGlobal g = {0, 0, SQLITE_DEFAULT_SECTOR_SIZE, 0, 0};
 */
 static int sqlite3CrashTestEnable = 0;
 
-static void *crash_malloc(int nByte){
-  return (void *)Tcl_Alloc((size_t)nByte);
-}
-static void crash_free(void *p){
-  Tcl_Free(p);
-}
-static void *crash_realloc(void *p, int n){
-  return (void *)Tcl_Realloc(p, (size_t)n);
-}
-
-/*
-** Wrapper around the sqlite3OsWrite() function that avoids writing to the
-** 512 byte block begining at offset PENDING_BYTE.
-*/
-static int writeDbFile(CrashFile *p, u8 *z, i64 iAmt, i64 iOff){
-  int rc = SQLITE_OK;
-  int iSkip = 0;
-  if( iOff==PENDING_BYTE && (p->flags&SQLITE_OPEN_MAIN_DB) ){
-    iSkip = 512;
-  }
-  if( (iAmt-iSkip)>0 ){
-    rc = sqlite3OsWrite(p->pRealFile, &z[iSkip], (int)(iAmt-iSkip), iOff+iSkip);
-  }
-  return rc;
-}
-
 /*
 ** Flush the write-list as if xSync() had been called on file handle
 ** pFile. If isCrash is true, simulate a crash.
@@ -211,7 +184,7 @@ static int writeListSync(CrashFile *pFile, int isCrash){
     int nWrite = 0;
     int iFinal;
     for(pWrite=g.pWriteList; pWrite; pWrite=pWrite->pNext) nWrite++;
-    sqlite3_randomness(sizeof(int), &iFinal);
+    sqlite3Randomness(sizeof(int), &iFinal);
     iFinal = ((iFinal<0)?-1*iFinal:iFinal)%nWrite;
     for(pWrite=g.pWriteList; iFinal>0; pWrite=pWrite->pNext) iFinal--;
     pFinal = pWrite;
@@ -237,7 +210,7 @@ static int writeListSync(CrashFile *pFile, int isCrash){
       }
     }else{
       char random;
-      sqlite3_randomness(1, &random);
+      sqlite3Randomness(1, &random);
 
       /* Do not select option 3 (sector trashing) if the IOCAP_ATOMIC flag 
       ** is set or this is an OsTruncate(), not an Oswrite().
@@ -276,8 +249,8 @@ static int writeListSync(CrashFile *pFile, int isCrash){
     switch( eAction ){
       case 1: {               /* Write out correctly */
         if( pWrite->zBuf ){
-          rc = writeDbFile(
-              pWrite->pFile, pWrite->zBuf, pWrite->nBuf, pWrite->iOffset
+          rc = sqlite3OsWrite(
+              pRealFile, pWrite->zBuf, pWrite->nBuf, pWrite->iOffset
           );
         }else{
           rc = sqlite3OsTruncate(pRealFile, pWrite->iOffset);
@@ -290,7 +263,7 @@ static int writeListSync(CrashFile *pFile, int isCrash){
           );
         }
 #endif
-        crash_free(pWrite);
+        sqlite3_free(pWrite);
         break;
       }
       case 2: {               /* Do nothing */
@@ -306,27 +279,27 @@ static int writeListSync(CrashFile *pFile, int isCrash){
       }
       case 3: {               /* Trash sectors */
         u8 *zGarbage;
-        int iFirst = (int)(pWrite->iOffset/g.iSectorSize);
-        int iLast = (int)((pWrite->iOffset+pWrite->nBuf-1)/g.iSectorSize);
+        int iFirst = (pWrite->iOffset/g.iSectorSize);
+        int iLast = (pWrite->iOffset+pWrite->nBuf-1)/g.iSectorSize;
 
         assert(pWrite->zBuf);
 
 #ifdef TRACE_CRASHTEST
-        printf("Trashing %d sectors @ %lld (sector %d) (%s)\n", 
-            1+iLast-iFirst, pWrite->iOffset, iFirst, pWrite->pFile->zName
+        printf("Trashing %d sectors @ sector %d (%s)\n", 
+            1+iLast-iFirst, iFirst, pWrite->pFile->zName
         );
 #endif
 
-        zGarbage = crash_malloc(g.iSectorSize);
+        zGarbage = sqlite3_malloc(g.iSectorSize);
         if( zGarbage ){
           sqlite3_int64 i;
           for(i=iFirst; rc==SQLITE_OK && i<=iLast; i++){
-            sqlite3_randomness(g.iSectorSize, zGarbage); 
-            rc = writeDbFile(
-              pWrite->pFile, zGarbage, g.iSectorSize, i*g.iSectorSize
+            sqlite3Randomness(g.iSectorSize, zGarbage); 
+            rc = sqlite3OsWrite(
+              pRealFile, zGarbage, g.iSectorSize, i*g.iSectorSize
             );
           }
-          crash_free(zGarbage);
+          sqlite3_free(zGarbage);
         }else{
           rc = SQLITE_NOMEM;
         }
@@ -365,11 +338,7 @@ static int writeListAppend(
 
   assert((zBuf && nBuf) || (!nBuf && !zBuf));
 
-  pNew = (WriteBuffer *)crash_malloc(sizeof(WriteBuffer) + nBuf);
-  if( pNew==0 ){
-    fprintf(stderr, "out of memory in the crash simulator\n");
-  }
-  memset(pNew, 0, sizeof(WriteBuffer)+nBuf);
+  pNew = (WriteBuffer *)sqlite3MallocZero(sizeof(WriteBuffer) + nBuf);
   pNew->iOffset = iOffset;
   pNew->nBuf = nBuf;
   pNew->pFile = (CrashFile *)pFile;
@@ -430,12 +399,12 @@ static int cfWrite(
 ){
   CrashFile *pCrash = (CrashFile *)pFile;
   if( iAmt+iOfst>pCrash->iSize ){
-    pCrash->iSize = (int)(iAmt+iOfst);
+    pCrash->iSize = iAmt+iOfst;
   }
   while( pCrash->iSize>pCrash->nData ){
     u8 *zNew;
     int nNew = (pCrash->nData*2) + 4096;
-    zNew = crash_realloc(pCrash->zData, nNew);
+    zNew = sqlite3_realloc(pCrash->zData, nNew);
     if( !zNew ){
       return SQLITE_NOMEM;
     }
@@ -454,7 +423,7 @@ static int cfTruncate(sqlite3_file *pFile, sqlite_int64 size){
   CrashFile *pCrash = (CrashFile *)pFile;
   assert(size>=0);
   if( pCrash->iSize>size ){
-    pCrash->iSize = (int)size;
+    pCrash->iSize = size;
   }
   return writeListAppend(pFile, size, 0, 0);
 }
@@ -468,23 +437,15 @@ static int cfSync(sqlite3_file *pFile, int flags){
 
   const char *zName = pCrash->zName;
   const char *zCrashFile = g.zCrashFile;
-  int nName = (int)strlen(zName);
-  int nCrashFile = (int)strlen(zCrashFile);
+  int nName = strlen(zName);
+  int nCrashFile = strlen(zCrashFile);
 
   if( nCrashFile>0 && zCrashFile[nCrashFile-1]=='*' ){
     nCrashFile--;
     if( nName>nCrashFile ) nName = nCrashFile;
   }
 
-#ifdef TRACE_CRASHTEST
-  printf("cfSync(): nName = %d, nCrashFile = %d, zName = %s, zCrashFile = %s\n",
-         nName, nCrashFile, zName, zCrashFile);
-#endif
-
   if( nName==nCrashFile && 0==memcmp(zName, zCrashFile, nName) ){
-#ifdef TRACE_CRASHTEST
-    printf("cfSync(): name matched, g.iCrash = %d\n", g.iCrash);
-#endif
     if( (--g.iCrash)==0 ) isCrash = 1;
   }
 
@@ -509,20 +470,10 @@ static int cfLock(sqlite3_file *pFile, int eLock){
 static int cfUnlock(sqlite3_file *pFile, int eLock){
   return sqlite3OsUnlock(((CrashFile *)pFile)->pRealFile, eLock);
 }
-static int cfCheckReservedLock(sqlite3_file *pFile, int *pResOut){
-  return sqlite3OsCheckReservedLock(((CrashFile *)pFile)->pRealFile, pResOut);
+static int cfCheckReservedLock(sqlite3_file *pFile){
+  return sqlite3OsCheckReservedLock(((CrashFile *)pFile)->pRealFile);
 }
 static int cfFileControl(sqlite3_file *pFile, int op, void *pArg){
-  if( op==SQLITE_FCNTL_SIZE_HINT ){
-    CrashFile *pCrash = (CrashFile *)pFile;
-    i64 nByte = *(i64 *)pArg;
-    if( nByte>pCrash->iSize ){
-      if( SQLITE_OK==writeListAppend(pFile, nByte, 0, 0) ){
-        pCrash->iSize = (int)nByte;
-      }
-    }
-    return SQLITE_OK;
-  }
   return sqlite3OsFileControl(((CrashFile *)pFile)->pRealFile, op, pArg);
 }
 
@@ -538,30 +489,8 @@ static int cfDeviceCharacteristics(sqlite3_file *pFile){
   return g.iDeviceCharacteristics;
 }
 
-/*
-** Pass-throughs for WAL support.
-*/
-static int cfShmLock(sqlite3_file *pFile, int ofst, int n, int flags){
-  return sqlite3OsShmLock(((CrashFile*)pFile)->pRealFile, ofst, n, flags);
-}
-static void cfShmBarrier(sqlite3_file *pFile){
-  sqlite3OsShmBarrier(((CrashFile*)pFile)->pRealFile);
-}
-static int cfShmUnmap(sqlite3_file *pFile, int delFlag){
-  return sqlite3OsShmUnmap(((CrashFile*)pFile)->pRealFile, delFlag);
-}
-static int cfShmMap(
-  sqlite3_file *pFile,            /* Handle open on database file */
-  int iRegion,                    /* Region to retrieve */
-  int sz,                         /* Size of regions */
-  int w,                          /* True to extend file if necessary */
-  void volatile **pp              /* OUT: Mapped memory */
-){
-  return sqlite3OsShmMap(((CrashFile*)pFile)->pRealFile, iRegion, sz, w, pp);
-}
-
 static const sqlite3_io_methods CrashFileVtab = {
-  2,                            /* iVersion */
+  1,                            /* iVersion */
   cfClose,                      /* xClose */
   cfRead,                       /* xRead */
   cfWrite,                      /* xWrite */
@@ -573,11 +502,7 @@ static const sqlite3_io_methods CrashFileVtab = {
   cfCheckReservedLock,          /* xCheckReservedLock */
   cfFileControl,                /* xFileControl */
   cfSectorSize,                 /* xSectorSize */
-  cfDeviceCharacteristics,      /* xDeviceCharacteristics */
-  cfShmMap,                     /* xShmMap */
-  cfShmLock,                    /* xShmLock */
-  cfShmBarrier,                 /* xShmBarrier */
-  cfShmUnmap                    /* xShmUnmap */
+  cfDeviceCharacteristics       /* xDeviceCharacteristics */
 };
 
 /*
@@ -618,30 +543,13 @@ static int cfOpen(
     pWrapper->pRealFile = pReal;
     rc = sqlite3OsFileSize(pReal, &iSize);
     pWrapper->iSize = (int)iSize;
-    pWrapper->flags = flags;
   }
   if( rc==SQLITE_OK ){
-    pWrapper->nData = (int)(4096 + pWrapper->iSize);
-    pWrapper->zData = crash_malloc(pWrapper->nData);
+    pWrapper->nData = (4096 + pWrapper->iSize);
+    pWrapper->zData = sqlite3_malloc(pWrapper->nData);
     if( pWrapper->zData ){
-      /* os_unix.c contains an assert() that fails if the caller attempts
-      ** to read data from the 512-byte locking region of a file opened
-      ** with the SQLITE_OPEN_MAIN_DB flag. This region of a database file
-      ** never contains valid data anyhow. So avoid doing such a read here.
-      **
-      ** UPDATE: It also contains an assert() verifying that each call
-      ** to the xRead() method reads less than 128KB of data.
-      */
-      const int isDb = (flags&SQLITE_OPEN_MAIN_DB);
-      i64 iOff;
-
       memset(pWrapper->zData, 0, pWrapper->nData);
-      for(iOff=0; iOff<pWrapper->iSize; iOff += 512){
-        int nRead = (int)(pWrapper->iSize - iOff);
-        if( nRead>512 ) nRead = 512;
-        if( isDb && iOff==PENDING_BYTE ) continue;
-        rc = sqlite3OsRead(pReal, &pWrapper->zData[iOff], nRead, iOff);
-      }
+      rc = sqlite3OsRead(pReal, pWrapper->zData, pWrapper->iSize, 0); 
     }else{
       rc = SQLITE_NOMEM;
     }
@@ -656,14 +564,13 @@ static int cfDelete(sqlite3_vfs *pCfVfs, const char *zPath, int dirSync){
   sqlite3_vfs *pVfs = (sqlite3_vfs *)pCfVfs->pAppData;
   return pVfs->xDelete(pVfs, zPath, dirSync);
 }
-static int cfAccess(
-  sqlite3_vfs *pCfVfs, 
-  const char *zPath, 
-  int flags, 
-  int *pResOut
-){
+static int cfAccess(sqlite3_vfs *pCfVfs, const char *zPath, int flags){
   sqlite3_vfs *pVfs = (sqlite3_vfs *)pCfVfs->pAppData;
-  return pVfs->xAccess(pVfs, zPath, flags, pResOut);
+  return pVfs->xAccess(pVfs, zPath, flags);
+}
+static int cfGetTempname(sqlite3_vfs *pCfVfs, int nBufOut, char *zBufOut){
+  sqlite3_vfs *pVfs = (sqlite3_vfs *)pCfVfs->pAppData;
+  return pVfs->xGetTempname(pVfs, nBufOut, zBufOut);
 }
 static int cfFullPathname(
   sqlite3_vfs *pCfVfs, 
@@ -682,9 +589,9 @@ static void cfDlError(sqlite3_vfs *pCfVfs, int nByte, char *zErrMsg){
   sqlite3_vfs *pVfs = (sqlite3_vfs *)pCfVfs->pAppData;
   pVfs->xDlError(pVfs, nByte, zErrMsg);
 }
-static void (*cfDlSym(sqlite3_vfs *pCfVfs, void *pH, const char *zSym))(void){
+static void *cfDlSym(sqlite3_vfs *pCfVfs, void *pHandle, const char *zSymbol){
   sqlite3_vfs *pVfs = (sqlite3_vfs *)pCfVfs->pAppData;
-  return pVfs->xDlSym(pVfs, pH, zSym);
+  return pVfs->xDlSym(pVfs, pHandle, zSymbol);
 }
 static void cfDlClose(sqlite3_vfs *pCfVfs, void *pHandle){
   sqlite3_vfs *pVfs = (sqlite3_vfs *)pCfVfs->pAppData;
@@ -714,18 +621,17 @@ static int processDevSymArgs(
     char *zName;
     int iValue;
   } aFlag[] = {
-    { "atomic",              SQLITE_IOCAP_ATOMIC                },
-    { "atomic512",           SQLITE_IOCAP_ATOMIC512             },
-    { "atomic1k",            SQLITE_IOCAP_ATOMIC1K              },
-    { "atomic2k",            SQLITE_IOCAP_ATOMIC2K              },
-    { "atomic4k",            SQLITE_IOCAP_ATOMIC4K              },
-    { "atomic8k",            SQLITE_IOCAP_ATOMIC8K              },
-    { "atomic16k",           SQLITE_IOCAP_ATOMIC16K             },
-    { "atomic32k",           SQLITE_IOCAP_ATOMIC32K             },
-    { "atomic64k",           SQLITE_IOCAP_ATOMIC64K             },
-    { "sequential",          SQLITE_IOCAP_SEQUENTIAL            },
-    { "safe_append",         SQLITE_IOCAP_SAFE_APPEND           },
-    { "powersafe_overwrite", SQLITE_IOCAP_POWERSAFE_OVERWRITE   },
+    { "atomic",      SQLITE_IOCAP_ATOMIC      },
+    { "atomic512",   SQLITE_IOCAP_ATOMIC512   },
+    { "atomic1k",    SQLITE_IOCAP_ATOMIC1K    },
+    { "atomic2k",    SQLITE_IOCAP_ATOMIC2K    },
+    { "atomic4k",    SQLITE_IOCAP_ATOMIC4K    },
+    { "atomic8k",    SQLITE_IOCAP_ATOMIC8K    },
+    { "atomic16k",   SQLITE_IOCAP_ATOMIC16K   },
+    { "atomic32k",   SQLITE_IOCAP_ATOMIC32K   },
+    { "atomic64k",   SQLITE_IOCAP_ATOMIC64K   },
+    { "sequential",  SQLITE_IOCAP_SEQUENTIAL  },
+    { "safe_append", SQLITE_IOCAP_SAFE_APPEND },
     { 0, 0 }
   };
 
@@ -810,7 +716,7 @@ static int crashEnableCmd(
 ){
   int isEnable;
   static sqlite3_vfs crashVfs = {
-    2,                  /* iVersion */
+    1,                  /* iVersion */
     0,                  /* szOsFile */
     0,                  /* mxPathname */
     0,                  /* pNext */
@@ -820,6 +726,7 @@ static int crashEnableCmd(
     cfOpen,               /* xOpen */
     cfDelete,             /* xDelete */
     cfAccess,             /* xAccess */
+    cfGetTempname,        /* xGetTempName */
     cfFullPathname,       /* xFullPathname */
     cfDlOpen,             /* xDlOpen */
     cfDlError,            /* xDlError */
@@ -827,9 +734,7 @@ static int crashEnableCmd(
     cfDlClose,            /* xDlClose */
     cfRandomness,         /* xRandomness */
     cfSleep,              /* xSleep */
-    cfCurrentTime,        /* xCurrentTime */
-    0,                    /* xGetlastError */
-    0,                    /* xCurrentTimeInt64 */
+    cfCurrentTime         /* xCurrentTime */
   };
 
   if( objc!=2 ){
@@ -930,74 +835,23 @@ static int devSymObjCmd(
   int objc,
   Tcl_Obj *CONST objv[]
 ){
-  void devsym_register(int iDeviceChar, int iSectorSize);
+
+  extern int sqlite3_test_device_characteristics;
+  extern int sqlite3_test_sector_size;
 
   int iDc = -1;
   int iSectorSize = -1;
-
   if( processDevSymArgs(interp, objc-1, &objv[1], &iDc, &iSectorSize) ){
     return TCL_ERROR;
   }
-  devsym_register(iDc, iSectorSize);
 
-  return TCL_OK;
-}
-
-/*
-** tclcmd: register_jt_vfs ?-default? PARENT-VFS
-*/
-static int jtObjCmd(
-  void * clientData,
-  Tcl_Interp *interp,
-  int objc,
-  Tcl_Obj *CONST objv[]
-){
-  int jt_register(char *, int);
-  char *zParent = 0;
-
-  if( objc!=2 && objc!=3 ){
-    Tcl_WrongNumArgs(interp, 1, objv, "?-default? PARENT-VFS");
-    return TCL_ERROR;
+  if( iDc>=0 ){
+    sqlite3_test_device_characteristics = iDc;
   }
-  zParent = Tcl_GetString(objv[1]);
-  if( objc==3 ){
-    if( strcmp(zParent, "-default") ){
-      Tcl_AppendResult(interp, 
-          "bad option \"", zParent, "\": must be -default", 0
-      );
-      return TCL_ERROR;
-    }
-    zParent = Tcl_GetString(objv[2]);
+  if( iSectorSize>=0 ){
+    sqlite3_test_sector_size = iSectorSize;
   }
 
-  if( !(*zParent) ){
-    zParent = 0;
-  }
-  if( jt_register(zParent, objc==3) ){
-    Tcl_AppendResult(interp, "Error in jt_register", 0);
-    return TCL_ERROR;
-  }
-
-  return TCL_OK;
-}
-
-/*
-** tclcmd: unregister_jt_vfs
-*/
-static int jtUnregisterObjCmd(
-  void * clientData,
-  Tcl_Interp *interp,
-  int objc,
-  Tcl_Obj *CONST objv[]
-){
-  void jt_unregister(void);
-
-  if( objc!=1 ){
-    Tcl_WrongNumArgs(interp, 1, objv, "");
-    return TCL_ERROR;
-  }
-
-  jt_unregister();
   return TCL_OK;
 }
 
@@ -1011,8 +865,6 @@ int Sqlitetest6_Init(Tcl_Interp *interp){
   Tcl_CreateObjCommand(interp, "sqlite3_crash_enable", crashEnableCmd, 0, 0);
   Tcl_CreateObjCommand(interp, "sqlite3_crashparams", crashParamsObjCmd, 0, 0);
   Tcl_CreateObjCommand(interp, "sqlite3_simulate_device", devSymObjCmd, 0, 0);
-  Tcl_CreateObjCommand(interp, "register_jt_vfs", jtObjCmd, 0, 0);
-  Tcl_CreateObjCommand(interp, "unregister_jt_vfs", jtUnregisterObjCmd, 0, 0);
 #endif
   return TCL_OK;
 }
