@@ -26,8 +26,8 @@
 **   from sqlite3_malloc(). Any attempt to create a temporary database file 
 **   will fail (SQLITE_IOERR). To prevent SQLite from attempting this,
 **   it should be configured to store all temporary database files in 
-**   main memory (see pragma "temp_store" or the SQLITE_TEMP_STORE compile 
-**   time option).
+**   main memory (see pragma "temp_store" or the TEMP_STORE compile time
+**   option).
 **
 ** ASSUMPTIONS:
 **
@@ -79,7 +79,7 @@
 **   time using this demo vfs.
 */
 
-#include "sqlite3.h"
+#include <sqlite3.h>
 #include <assert.h>
 #include <string.h>
 
@@ -136,7 +136,7 @@ static int fsSync(sqlite3_file*, int flags);
 static int fsFileSize(sqlite3_file*, sqlite3_int64 *pSize);
 static int fsLock(sqlite3_file*, int);
 static int fsUnlock(sqlite3_file*, int);
-static int fsCheckReservedLock(sqlite3_file*, int *pResOut);
+static int fsCheckReservedLock(sqlite3_file*);
 static int fsFileControl(sqlite3_file*, int op, void *pArg);
 static int fsSectorSize(sqlite3_file*);
 static int fsDeviceCharacteristics(sqlite3_file*);
@@ -152,7 +152,7 @@ static int tmpSync(sqlite3_file*, int flags);
 static int tmpFileSize(sqlite3_file*, sqlite3_int64 *pSize);
 static int tmpLock(sqlite3_file*, int);
 static int tmpUnlock(sqlite3_file*, int);
-static int tmpCheckReservedLock(sqlite3_file*, int *pResOut);
+static int tmpCheckReservedLock(sqlite3_file*);
 static int tmpFileControl(sqlite3_file*, int op, void *pArg);
 static int tmpSectorSize(sqlite3_file*);
 static int tmpDeviceCharacteristics(sqlite3_file*);
@@ -162,11 +162,12 @@ static int tmpDeviceCharacteristics(sqlite3_file*);
 */
 static int fsOpen(sqlite3_vfs*, const char *, sqlite3_file*, int , int *);
 static int fsDelete(sqlite3_vfs*, const char *zName, int syncDir);
-static int fsAccess(sqlite3_vfs*, const char *zName, int flags, int *);
+static int fsAccess(sqlite3_vfs*, const char *zName, int flags);
+static int fsGetTempname(sqlite3_vfs*, int nOut, char *zOut);
 static int fsFullPathname(sqlite3_vfs*, const char *zName, int nOut,char *zOut);
 static void *fsDlOpen(sqlite3_vfs*, const char *zFilename);
 static void fsDlError(sqlite3_vfs*, int nByte, char *zErrMsg);
-static void (*fsDlSym(sqlite3_vfs*,void*, const char *zSymbol))(void);
+static void *fsDlSym(sqlite3_vfs*,void*, const char *zSymbol);
 static void fsDlClose(sqlite3_vfs*, void*);
 static int fsRandomness(sqlite3_vfs*, int nByte, char *zOut);
 static int fsSleep(sqlite3_vfs*, int microseconds);
@@ -191,6 +192,7 @@ static fs_vfs_t fs_vfs = {
     fsOpen,                                     /* xOpen */
     fsDelete,                                   /* xDelete */
     fsAccess,                                   /* xAccess */
+    fsGetTempname,                              /* xGetTempName */
     fsFullPathname,                             /* xFullPathname */
     fsDlOpen,                                   /* xDlOpen */
     fsDlError,                                  /* xDlError */
@@ -198,8 +200,7 @@ static fs_vfs_t fs_vfs = {
     fsDlClose,                                  /* xDlClose */
     fsRandomness,                               /* xRandomness */
     fsSleep,                                    /* xSleep */
-    fsCurrentTime,                              /* xCurrentTime */
-    0                                           /* xCurrentTimeInt64 */
+    fsCurrentTime                               /* xCurrentTime */
   }, 
   0,                                            /* pFileList */
   0                                             /* pParent */
@@ -218,11 +219,7 @@ static sqlite3_io_methods fs_io_methods = {
   fsCheckReservedLock,          /* xCheckReservedLock */
   fsFileControl,                /* xFileControl */
   fsSectorSize,                 /* xSectorSize */
-  fsDeviceCharacteristics,      /* xDeviceCharacteristics */
-  0,                            /* xShmMap */
-  0,                            /* xShmLock */
-  0,                            /* xShmBarrier */
-  0                             /* xShmUnmap */
+  fsDeviceCharacteristics       /* xDeviceCharacteristics */
 };
 
 
@@ -239,11 +236,7 @@ static sqlite3_io_methods tmp_io_methods = {
   tmpCheckReservedLock,         /* xCheckReservedLock */
   tmpFileControl,               /* xFileControl */
   tmpSectorSize,                /* xSectorSize */
-  tmpDeviceCharacteristics,     /* xDeviceCharacteristics */
-  0,                            /* xShmMap */
-  0,                            /* xShmLock */
-  0,                            /* xShmBarrier */
-  0                             /* xShmUnmap */
+  tmpDeviceCharacteristics      /* xDeviceCharacteristics */
 };
 
 /* Useful macros used in several places */
@@ -288,7 +281,7 @@ static int tmpWrite(
 ){
   tmp_file *pTmp = (tmp_file *)pFile;
   if( (iAmt+iOfst)>pTmp->nAlloc ){
-    int nNew = (int)(2*(iAmt+iOfst+pTmp->nAlloc));
+    int nNew = 2*(iAmt+iOfst+pTmp->nAlloc);
     char *zNew = sqlite3_realloc(pTmp->zAlloc, nNew);
     if( !zNew ){
       return SQLITE_NOMEM;
@@ -297,7 +290,7 @@ static int tmpWrite(
     pTmp->nAlloc = nNew;
   }
   memcpy(&pTmp->zAlloc[iOfst], zBuf, iAmt);
-  pTmp->nSize = (int)MAX(pTmp->nSize, iOfst+iAmt);
+  pTmp->nSize = MAX(pTmp->nSize, iOfst+iAmt);
   return SQLITE_OK;
 }
 
@@ -306,7 +299,7 @@ static int tmpWrite(
 */
 static int tmpTruncate(sqlite3_file *pFile, sqlite_int64 size){
   tmp_file *pTmp = (tmp_file *)pFile;
-  pTmp->nSize = (int)MIN(pTmp->nSize, size);
+  pTmp->nSize = MIN(pTmp->nSize, size);
   return SQLITE_OK;
 }
 
@@ -343,8 +336,7 @@ static int tmpUnlock(sqlite3_file *pFile, int eLock){
 /*
 ** Check if another file-handle holds a RESERVED lock on a tmp-file.
 */
-static int tmpCheckReservedLock(sqlite3_file *pFile, int *pResOut){
-  *pResOut = 0;
+static int tmpCheckReservedLock(sqlite3_file *pFile){
   return SQLITE_OK;
 }
 
@@ -418,7 +410,7 @@ static int fsRead(
     /* Journal file. */
     int iRem = iAmt;
     int iBuf = 0;
-    int ii = (int)iOfst;
+    int ii = iOfst;
     while( iRem>0 && rc==SQLITE_OK ){
       int iRealOff = pReal->nBlob - BLOCKSIZE*((ii/BLOCKSIZE)+1) + ii%BLOCKSIZE;
       int iRealAmt = MIN(iRem, BLOCKSIZE - (iRealOff%BLOCKSIZE));
@@ -453,14 +445,14 @@ static int fsWrite(
     }else{
       rc = pF->pMethods->xWrite(pF, zBuf, iAmt, iOfst+BLOCKSIZE);
       if( rc==SQLITE_OK ){
-        pReal->nDatabase = (int)MAX(pReal->nDatabase, iAmt+iOfst);
+        pReal->nDatabase = MAX(pReal->nDatabase, iAmt+iOfst);
       }
     }
   }else{
     /* Journal file. */
     int iRem = iAmt;
     int iBuf = 0;
-    int ii = (int)iOfst;
+    int ii = iOfst;
     while( iRem>0 && rc==SQLITE_OK ){
       int iRealOff = pReal->nBlob - BLOCKSIZE*((ii/BLOCKSIZE)+1) + ii%BLOCKSIZE;
       int iRealAmt = MIN(iRem, BLOCKSIZE - (iRealOff%BLOCKSIZE));
@@ -475,7 +467,7 @@ static int fsWrite(
       }
     }
     if( rc==SQLITE_OK ){
-      pReal->nJournal = (int)MAX(pReal->nJournal, iAmt+iOfst);
+      pReal->nJournal = MAX(pReal->nJournal, iAmt+iOfst);
     }
   }
 
@@ -489,9 +481,9 @@ static int fsTruncate(sqlite3_file *pFile, sqlite_int64 size){
   fs_file *p = (fs_file *)pFile;
   fs_real_file *pReal = p->pReal;
   if( p->eType==DATABASE_FILE ){
-    pReal->nDatabase = (int)MIN(pReal->nDatabase, size);
+    pReal->nDatabase = MIN(pReal->nDatabase, size);
   }else{
-    pReal->nJournal = (int)MIN(pReal->nJournal, size);
+    pReal->nJournal = MIN(pReal->nJournal, size);
   }
   return SQLITE_OK;
 }
@@ -551,9 +543,8 @@ static int fsUnlock(sqlite3_file *pFile, int eLock){
 /*
 ** Check if another file-handle holds a RESERVED lock on an fs-file.
 */
-static int fsCheckReservedLock(sqlite3_file *pFile, int *pResOut){
-  *pResOut = 0;
-  return SQLITE_OK;
+static int fsCheckReservedLock(sqlite3_file *pFile){
+  return 0;
 }
 
 /*
@@ -606,12 +597,11 @@ static int fsOpen(
   p->eType = eType;
 
   assert(strlen("-journal")==8);
-  nName = (int)strlen(zName)-((eType==JOURNAL_FILE)?8:0);
+  nName = strlen(zName)-((eType==JOURNAL_FILE)?8:0);
   pReal=pFsVfs->pFileList; 
   for(; pReal && strncmp(pReal->zName, zName, nName); pReal=pReal->pNext);
 
   if( !pReal ){
-    int real_flags = (flags&~(SQLITE_OPEN_MAIN_DB))|SQLITE_OPEN_TEMP_DB;
     sqlite3_int64 size;
     sqlite3_file *pRealFile;
     sqlite3_vfs *pParent = pFsVfs->pParent;
@@ -626,7 +616,7 @@ static int fsOpen(
     pReal->zName = zName;
     pReal->pFile = (sqlite3_file *)(&pReal[1]);
 
-    rc = pParent->xOpen(pParent, zName, pReal->pFile, real_flags, pOutFlags);
+    rc = pParent->xOpen(pParent, zName, pReal->pFile, flags, pOutFlags);
     if( rc!=SQLITE_OK ){
       goto open_out;
     }
@@ -641,7 +631,7 @@ static int fsOpen(
       pReal->nBlob = BLOBSIZE;
     }else{
       unsigned char zS[4];
-      pReal->nBlob = (int)size;
+      pReal->nBlob = size;
       rc = pRealFile->pMethods->xRead(pRealFile, zS, 4, 0);
       pReal->nDatabase = (zS[0]<<24)+(zS[1]<<16)+(zS[2]<<8)+zS[3];
       if( rc==SQLITE_OK ){
@@ -687,7 +677,7 @@ static int fsDelete(sqlite3_vfs *pVfs, const char *zPath, int dirSync){
   fs_vfs_t *pFsVfs = (fs_vfs_t *)pVfs;
   fs_real_file *pReal;
   sqlite3_file *pF;
-  int nName = (int)strlen(zPath) - 8;
+  int nName = strlen(zPath) - 8;
 
   assert(strlen("-journal")==8);
   assert(strcmp("-journal", &zPath[nName])==0);
@@ -708,20 +698,15 @@ static int fsDelete(sqlite3_vfs *pVfs, const char *zPath, int dirSync){
 ** Test for access permissions. Return true if the requested permission
 ** is available, or false otherwise.
 */
-static int fsAccess(
-  sqlite3_vfs *pVfs, 
-  const char *zPath, 
-  int flags, 
-  int *pResOut
-){
+static int fsAccess(sqlite3_vfs *pVfs, const char *zPath, int flags){
   fs_vfs_t *pFsVfs = (fs_vfs_t *)pVfs;
   fs_real_file *pReal;
   int isJournal = 0;
-  int nName = (int)strlen(zPath);
+  int nName = strlen(zPath);
 
   if( flags!=SQLITE_ACCESS_EXISTS ){
     sqlite3_vfs *pParent = ((fs_vfs_t *)pVfs)->pParent;
-    return pParent->xAccess(pParent, zPath, flags, pResOut);
+    return pParent->xAccess(pParent, zPath, flags);
   }
 
   assert(strlen("-journal")==8);
@@ -732,9 +717,18 @@ static int fsAccess(
 
   pReal = pFsVfs->pFileList; 
   for(; pReal && strncmp(pReal->zName, zPath, nName); pReal=pReal->pNext);
+  if( !pReal ) return 0;
+  return ((!isJournal||pReal->nJournal>0)?1:0);
+}
 
-  *pResOut = (pReal && (!isJournal || pReal->nJournal>0));
-  return SQLITE_OK;
+/*
+** Populate buffer zBufOut with a pathname suitable for use as a 
+** temporary file. zBufOut is guaranteed to point to a buffer of 
+** at least (FS_MAX_PATHNAME+1) bytes.
+*/
+static int fsGetTempname(sqlite3_vfs *pVfs, int nBufOut, char *zBufOut){
+  sqlite3_vfs *pParent = ((fs_vfs_t *)pVfs)->pParent;
+  return pParent->xGetTempname(pParent, nBufOut, zBufOut);
 }
 
 /*
@@ -773,9 +767,9 @@ static void fsDlError(sqlite3_vfs *pVfs, int nByte, char *zErrMsg){
 /*
 ** Return a pointer to the symbol zSymbol in the dynamic library pHandle.
 */
-static void (*fsDlSym(sqlite3_vfs *pVfs, void *pH, const char *zSym))(void){
+static void *fsDlSym(sqlite3_vfs *pVfs, void *pHandle, const char *zSymbol){
   sqlite3_vfs *pParent = ((fs_vfs_t *)pVfs)->pParent;
-  return pParent->xDlSym(pParent, pH, zSym);
+  return pParent->xDlSym(pParent, pHandle, zSymbol);
 }
 
 /*
@@ -817,7 +811,7 @@ static int fsCurrentTime(sqlite3_vfs *pVfs, double *pTimeOut){
 ** true, the fs vfs becomes the new default vfs. It is the only publicly
 ** available function in this file.
 */
-int fs_register(void){
+int fs_register(){
   if( fs_vfs.pParent ) return SQLITE_OK;
   fs_vfs.pParent = sqlite3_vfs_find(0);
   fs_vfs.base.mxPathname = fs_vfs.pParent->mxPathname;
@@ -828,3 +822,4 @@ int fs_register(void){
 #ifdef SQLITE_TEST
   int SqlitetestOnefile_Init() {return fs_register();}
 #endif
+
